@@ -8,7 +8,7 @@
 
 # Package name      : perl5
 # Source directory  : .
-# Configuration time: Sat May 22 00:43:12 EET DST 1999
+# Configuration time: Sun Jul 25 17:08:22 EET DST 1999
 # Configured by     : jhi
 # Target system     : osf1 alpha.hut.fi v4.0 878 alpha 
 
@@ -30,10 +30,10 @@ afs='false'
 alignbytes='8'
 ansi2knr=''
 aphostname=''
-apiversion='5.00556'
+apiversion='5.00557'
 ar='ar'
-archlib='/opt/perl/lib/5.00556/alpha-dec_osf-thread'
-archlibexp='/opt/perl/lib/5.00556/alpha-dec_osf-thread'
+archlib='/opt/perl/lib/5.00557/alpha-dec_osf-thread'
+archlibexp='/opt/perl/lib/5.00557/alpha-dec_osf-thread'
 archname64=''
 archname='alpha-dec_osf-thread'
 archobjs=''
@@ -50,12 +50,12 @@ castflags='0'
 cat='cat'
 cc='cc'
 cccdlflags=' '
-ccdlflags='  -Wl,-rpath,/opt/perl/lib/5.00556/alpha-dec_osf-thread/CORE'
+ccdlflags='  -Wl,-rpath,/opt/perl/lib/5.00557/alpha-dec_osf-thread/CORE'
 ccflags='-pthread -std -DLANGUAGE_C'
 ccsymbols='__LANGUAGE_C__=1 _LONGLONG=1 LANGUAGE_C=1 SYSTYPE_BSD=1'
 cf_by='jhi'
 cf_email='yourname@yourhost.yourplace.com'
-cf_time='Sat May 22 00:43:12 EET DST 1999'
+cf_time='Sun Jul 25 17:08:22 EET DST 1999'
 chgrp=''
 chmod=''
 chown=''
@@ -121,6 +121,7 @@ d_endnent='define'
 d_endpent='define'
 d_endpwent='define'
 d_endsent='define'
+d_endspent='undef'
 d_eofnblk='define'
 d_eunice='undef'
 d_fchmod='define'
@@ -180,6 +181,8 @@ d_getsbyname='define'
 d_getsbyport='define'
 d_getsent='define'
 d_getservprotos='define'
+d_getspent='undef'
+d_getspnam='undef'
 d_gettimeod='define'
 d_gnulibc='undef'
 d_grpasswd='define'
@@ -301,6 +304,7 @@ d_setrgid='define'
 d_setruid='define'
 d_setsent='define'
 d_setsid='define'
+d_setspent='undef'
 d_setvbuf='define'
 d_sfio='undef'
 d_shm='define'
@@ -354,6 +358,7 @@ d_tzname='define'
 d_umask='define'
 d_uname='define'
 d_union_semun='undef'
+d_vendorlib='undef'
 d_vfork='undef'
 d_void_closedir='undef'
 d_voidsig='define'
@@ -375,7 +380,7 @@ dlext='so'
 dlsrc='dl_dlopen.xs'
 doublesize='8'
 drand01='drand48()'
-dynamic_ext='B ByteLoader DB_File Data/Dumper Devel/Peek Fcntl IO IPC/SysV NDBM_File ODBM_File Opcode POSIX SDBM_File Socket Thread attrs re'
+dynamic_ext='B ByteLoader DB_File Data/Dumper Devel/DProf Devel/Peek Fcntl IO IPC/SysV NDBM_File ODBM_File Opcode POSIX SDBM_File Socket Thread attrs re'
 eagain='EAGAIN'
 ebcdic='undef'
 echo='echo'
@@ -384,7 +389,7 @@ emacs=''
 eunicefix=':'
 exe_ext=''
 expr='expr'
-extensions='B ByteLoader DB_File Data/Dumper Devel/Peek Fcntl IO IPC/SysV NDBM_File ODBM_File Opcode POSIX SDBM_File Socket Thread attrs re Errno'
+extensions='B ByteLoader DB_File Data/Dumper Devel/DProf Devel/Peek Fcntl IO IPC/SysV NDBM_File ODBM_File Opcode POSIX SDBM_File Socket Thread attrs re Errno'
 fflushNULL='define'
 fflushall='undef'
 find=''
@@ -437,6 +442,8 @@ i_pwd='define'
 i_rpcsvcdbm='undef'
 i_sfio='undef'
 i_sgtty='undef'
+i_shadow='undef'
+i_socks='undef'
 i_stdarg='define'
 i_stddef='define'
 i_stdlib='define'
@@ -476,29 +483,34 @@ i_vfork='undef'
 ignore_versioned_solibs=''
 incpath=''
 inews=''
-installarchlib='/opt/perl/lib/5.00556/alpha-dec_osf-thread'
+installarchlib='/opt/perl/lib/5.00557/alpha-dec_osf-thread'
 installbin='/opt/perl/bin'
 installman1dir='/opt/perl/man/man1'
 installman3dir='/opt/perl/man/man3'
-installprivlib='/opt/perl/lib/5.00556'
+installprefix='/opt/perl'
+installprefixexp='/opt/perl'
+installprivlib='/opt/perl/lib/5.00557'
 installscript='/opt/perl/bin'
-installsitearch='/opt/perl/lib/site_perl/5.00556/alpha-dec_osf-thread'
-installsitelib='/opt/perl/lib/site_perl/5.00556'
+installsitearch='/opt/perl/lib/site_perl/5.00557/alpha-dec_osf-thread'
+installsitelib='/opt/perl/lib/site_perl'
+installstyle='lib'
 installusrbinperl='define'
+installvendorlib=''
 intsize='4'
-known_extensions='B ByteLoader DB_File Data/Dumper Devel/Peek Fcntl GDBM_File IO IPC/SysV NDBM_File ODBM_File Opcode POSIX SDBM_File Socket Thread attrs re'
+known_extensions='B ByteLoader DB_File Data/Dumper Devel/DProf Devel/Peek Fcntl GDBM_File IO IPC/SysV NDBM_File ODBM_File Opcode POSIX SDBM_File Socket Thread attrs re'
 ksh=''
 large=''
 ld='ld'
 lddlflags='-shared -expect_unresolved "*" -msym -s'
 ldflags=''
+ldlibpthname='LD_LIBRARY_PATH'
 less='less'
 lib_ext='.a'
 libc='/usr/shlib/libc.so'
 libperl='libperl.so'
 libpth='/usr/shlib /usr/ccs/lib /usr/lib/cmplrs/cc /usr/lib /var/shlib'
 libs='-lgdbm -ldbm -ldb -lm -lrt -lpthread -lexc'
-libswanted='sfio socket inet nsl nm gdbm dbm db malloc dld ld sun m rt cposix posix ndir dir crypt ucb BSD x pthread exc'
+libswanted='sfio socket inet nsl nm gdbm dbm db malloc dld ld sun m rt cposix posix ndir dir crypt sec ucb BSD x pthread exc'
 line=''
 lint=''
 lkflags=''
@@ -574,8 +586,8 @@ pmake=''
 pr=''
 prefix='/opt/perl'
 prefixexp='/opt/perl'
-privlib='/opt/perl/lib/5.00556'
-privlibexp='/opt/perl/lib/5.00556'
+privlib='/opt/perl/lib/5.00557'
+privlibexp='/opt/perl/lib/5.00557'
 prototype='define'
 ptrsize='8'
 randbits='48'
@@ -607,10 +619,12 @@ sig_name_init='"ZERO", "HUP", "INT", "QUIT", "ILL", "TRAP", "ABRT", "EMT", "FPE"
 sig_num='0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 6 6 16 20 23 23 23 29 48 '
 sig_num_init='0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 6, 6, 16, 20, 23, 23, 23, 29, 48, 0'
 signal_t='void'
-sitearch='/opt/perl/lib/site_perl/5.00556/alpha-dec_osf-thread'
-sitearchexp='/opt/perl/lib/site_perl/5.00556/alpha-dec_osf-thread'
-sitelib='/opt/perl/lib/site_perl/5.00556'
-sitelibexp='/opt/perl/lib/site_perl/5.00556'
+sitearch='/opt/perl/lib/site_perl/5.00557/alpha-dec_osf-thread'
+sitearchexp='/opt/perl/lib/site_perl/5.00557/alpha-dec_osf-thread'
+sitelib='/opt/perl/lib/site_perl'
+sitelibexp='/opt/perl/lib/site_perl'
+siteprefix='/opt/perl'
+siteprefixexp='/opt/perl'
 sizetype='size_t'
 sleep=''
 smail=''
@@ -636,7 +650,7 @@ stdio_ptr='((fp)->_ptr)'
 stdio_stream_array='_iob'
 strings='/usr/include/string.h'
 submit=''
-subversion='56'
+subversion='57'
 sysman='/usr/man/man1'
 tail=''
 tar=''
@@ -662,11 +676,17 @@ useperlio='undef'
 useposix='true'
 usesfio='false'
 useshrplib='true'
+usesocks='undef'
 usethreads='define'
+usevendorprefix='undef'
 usevfork='false'
 usrinc='/usr/include'
 uuname=''
-version='5.00556'
+vendorlib=''
+vendorlibexp=''
+vendorprefix=''
+vendorprefixexp=''
+version='5.00557'
 vi=''
 voidflags='15'
 xlibpth='/usr/lib/386 /lib/386'
@@ -688,7 +708,7 @@ config_arg9='-Dmyhostname=yourhost'
 config_arg10='-dE'
 PERL_REVISION=5
 PERL_VERSION=5
-PERL_SUBVERSION=56
+PERL_SUBVERSION=57
 CONFIGDOTSH=true
 # Variables propagated from previous config.sh file.
 pp_sys_cflags='ccflags="$ccflags -DNO_EFF_ONLY_OK"'
