@@ -65,6 +65,7 @@ my $anonsub_index = 0;
 my $initsub_index = 0;
 
 my %symtable;
+my %xsub;
 my $warn_undefined_syms;
 my $verbose;
 my %unused_sub_packages;
@@ -82,7 +83,7 @@ my ($init, $decl, $symsect, $binopsect, $condopsect, $copsect,
     $gvopsect, $listopsect, $logopsect, $loopsect, $opsect, $pmopsect,
     $pvopsect, $svopsect, $unopsect, $svsect, $xpvsect, $xpvavsect,
     $xpvhvsect, $xpvcvsect, $xpvivsect, $xpvnvsect, $xpvmgsect, $xpvlvsect,
-    $xrvsect, $xpvbmsect, $xpviosect, $bootstrap);
+    $xrvsect, $xpvbmsect, $xpviosect );
 
 sub walk_and_save_optree;
 my $saveoptree_callback = \&walk_and_save_optree;
@@ -233,18 +234,6 @@ sub B::LOGOP::save {
     savesym($op, sprintf("(OP*)&logop_list[%d]", $logopsect->index));
 }
 
-sub B::CONDOP::save {
-    my ($op, $level) = @_;
-    my $sym = objsym($op);
-    return $sym if defined $sym;
-    $condopsect->add(sprintf("s\\_%x, s\\_%x, %s, %u, %u, %u, 0x%x, 0x%x, s\\_%x, s\\_%x, s\\_%x",
-			     ${$op->next}, ${$op->sibling}, $op->ppaddr,
-			     $op->targ, $op->type, $op_seq, $op->flags,
-			     $op->private, ${$op->first}, ${$op->true},
-			     ${$op->false}));
-    savesym($op, sprintf("(OP*)&condop_list[%d]", $condopsect->index));
-}
-
 sub B::LOOP::save {
     my ($op, $level) = @_;
     my $sym = objsym($op);
@@ -330,7 +319,7 @@ sub B::PMOP::save {
 	# OP_PUSHRE (a mutated version of OP_MATCH for the regexp
 	# argument to a split) stores a GV in op_pmreplroot instead
 	# of a substitution syntax tree. We don't want to walk that...
-	if ($ppaddr eq "pp_pushre") {
+	if ($op->name eq "pushre") {
 	    $gvsym = $replroot->save;
 #	    warn "PMOP::save saving a pp_pushre with GV $gvsym\n"; # debug
 	    $replrootfield = 0;
@@ -551,7 +540,7 @@ sub B::PVMG::save_magic {
 	if ($len == HEf_SVKEY){
 		#The pointer is an SV*
 		$ptrsv=svref_2object($ptr)->save;
-		$init->add(sprintf("sv_magic((SV*)s\\_%x, (SV*)s\\_%x, %s, %s, %d);",
+		$init->add(sprintf("sv_magic((SV*)s\\_%x, (SV*)s\\_%x, %s,(char *) %s, %d);",
 			   $$sv, $$obj, cchar($type),$ptrsv,$len));
 	}else{
 		$init->add(sprintf("sv_magic((SV*)s\\_%x, (SV*)s\\_%x, %s, %s, %d);",
@@ -602,6 +591,16 @@ sub B::CV::save {
 	return $sym;
     }
     # Reserve a place in svsect and xpvcvsect and record indices
+    my $gv = $cv->GV;
+    my $cvstashname = $gv->STASH->NAME;
+    my $cvname = $gv->NAME;
+    my $root = $cv->ROOT;
+    my $cvxsub = $cv->XSUB;
+    if ($cvxsub) {
+	my $egv = $gv->EGV;
+	my $stashname = $egv->STASH->NAME;
+	$xsub{$stashname}='Static' unless  $xsub{$stashname};
+    }
     my $sv_ix = $svsect->index + 1;
     $svsect->add("svix$sv_ix");
     my $xpvcv_ix = $xpvcvsect->index + 1;
@@ -609,11 +608,6 @@ sub B::CV::save {
     # Save symbol now so that GvCV() doesn't recurse back to us via CvGV()
     $sym = savesym($cv, "&sv_list[$sv_ix]");
     warn sprintf("saving CV 0x%x as $sym\n", $$cv) if $debug_cv;
-    my $gv = $cv->GV;
-    my $cvstashname = $gv->STASH->NAME;
-    my $cvname = $gv->NAME;
-    my $root = $cv->ROOT;
-    my $cvxsub = $cv->XSUB;
     if (!$$root && !$cvxsub) {
 	if (try_autoload($cvstashname, $cvname)) {
 	    # Recalculate root and xsub
@@ -662,17 +656,6 @@ sub B::CV::save {
 			 $$padlist, $$cv) if $debug_cv;
 	}
     }
-    elsif ($cvxsub) {
-	$xsubany = sprintf("ANYINIT((void*)0x%x)", $cv->XSUBANY);
-	# Try to find out canonical name of XSUB function from EGV.
-	# XXX Doesn't work for XSUBs with PREFIX set (or anyone who
-	# calls newXS() manually with weird arguments).
-	my $egv = $gv->EGV;
-	my $stashname = $egv->STASH->NAME;
-	$stashname =~ s/::/__/g;
-	$xsub = sprintf("XS_%s_%s", $stashname, $egv->NAME);
-	$decl->add("void $xsub _((CV*));");
-    }
     else {
 	warn sprintf("No definition for sub %s::%s (unable to autoload)\n",
 		     $cvstashname, $cvname); # debug
@@ -772,11 +755,21 @@ sub B::GV::save {
 #	    warn "GV::save \%$name\n"; # debug
 	}
 	my $gvcv = $gv->CV;
-	if ($$gvcv && !$skip_cv) {
-	    $gvcv->save;
-	    $init->add(sprintf("GvCV($sym) = (CV*)s\\_%x;", $$gvcv));
-#	    warn "GV::save &$name\n"; # debug
-	}
+	if ($$gvcv && !$skip_cv && !$gvcv->XSUB) { #not XSUB
+            $gvcv->save;
+            $init->add(sprintf("GvCV($sym) = (CV*)s\\_%x;", $$gvcv));
+#           warn "GV::save &$name\n"; # debug
+        }elsif ($$gvcv && $gvcv->XSUB && $name ne 
+               (my $origname=cstring($gvcv->GV->EGV->STASH->NAME .
+		 "::" . $gvcv->GV->EGV->NAME))) { #XSUB alias
+
+                $init->add("{ CV *cv;");
+                $init->add("\tcv=GvCV(gv_fetchpv($origname,FALSE,SVt_PV));");
+                $init->add("\tGvCV($sym)=cv;");
+                $init->add("\tSvREFCNT_inc((SV *)cv);");
+                $init->add("}");
+
+        }     
 	my $gvfilegv = $gv->FILEGV;
 	if ($$gvfilegv) {
 	    $gvfilegv->save;
@@ -939,7 +932,6 @@ sub output_all {
 		    $loopsect, $copsect, $svsect, $xpvsect,
 		    $xpvavsect, $xpvhvsect, $xpvcvsect, $xpvivsect, $xpvnvsect,
 		    $xpvmgsect, $xpvlvsect, $xrvsect, $xpvbmsect, $xpviosect);
-    $bootstrap->output(\*STDOUT, "/* bootstrap %s */\n");
     $symsect->output(\*STDOUT, "#define %s\n");
     print "\n";
     output_declarations();
@@ -968,6 +960,8 @@ sub output_all {
 static int $init_name()
 {
 	dTHR;
+	dTARG;
+	djSP;
 EOT
     $init->output(\*STDOUT, "\t%s\n");
     print "\treturn 0;\n}\n";
@@ -1002,7 +996,7 @@ typedef struct {
     HV *	xcv_stash;
     OP *	xcv_start;
     OP *	xcv_root;
-    void      (*xcv_xsub) _((CV*));
+    void      (*xcv_xsub) (CV*);
     void *	xcv_xsubany;
     GV *	xcv_gv;
     GV *	xcv_filegv;
@@ -1037,12 +1031,12 @@ sub output_boilerplate {
 #include "perl.h"
 
 /* Workaround for mapstart: the only op which needs a different ppaddr */
-#undef pp_mapstart
-#define pp_mapstart pp_grepstart
+#undef Perl_pp_mapstart
+#define Perl_pp_mapstart Perl_pp_grepstart
 #define XS_DynaLoader_boot_DynaLoader boot_DynaLoader
-EXTERN_C void boot_DynaLoader _((CV* cv));
+EXTERN_C void boot_DynaLoader (CV* cv);
 
-static void xs_init _((void));
+static void xs_init (void);
 static PerlInterpreter *my_perl;
 EOT
 }
@@ -1121,12 +1115,35 @@ static void
 xs_init()
 {
     char *file = __FILE__;
-    dXSUB_SYS;
-        newXS("DynaLoader::boot_DynaLoader", boot_DynaLoader, file);
-}                                                                              
+    dTARG;
+    djSP;
 EOT
-}
+    print "\n#ifdef USE_DYNAMIC_LOADING";
+    print qq/\n\tnewXS("DynaLoader::boot_DynaLoader", boot_DynaLoader, file);/;
+    print "\n#endif\n" ;
+    delete $xsub{'DynaLoader'}; 
+    delete $xsub{'UNIVERSAL'}; 
+    print("/* bootstrapping code*/\nSAVETMPS;\n");
+    print("\ttarg=sv_newmortal();\n");
+    foreach my $stashname (keys %xsub ){
+	my $stashxsub=$stashname;
+	$stashxsub  =~ s/::/__/g; 
+	if ($xsub{$stashname} eq 'Dynamic') {
+           print "#ifdef DYNALOADER_BOOTSTRAP\n";
+	   warn "bootstrapping $stashname added to xs_init\n";
+	   print qq/\n\t{\n\tchar *args[]={"$stashxsub", NULL};/;
+           print qq/\n\t\tperl_call_argv("${stashxsub}::bootstrap",G_DISCARD,args);\n\t}/;
+           print "\n#else\n";
+	}
+	print "\tPUSHMARK(sp);\n";
+   	print qq/\tXPUSHp("$stashname",strlen("$stashname")+1);\n/;
+	print "\tboot_$stashxsub(NULL);\n";
+        print "#endif\n" if ($xsub{$stashname} eq 'Dynamic');
+    }
 
+    print("\tFREETMPS;\n/* end bootstrapping code */\n");
+    print "\n}";
+}
 sub dump_symtable {
     # For debugging
     my ($sym, $val);
@@ -1167,10 +1184,10 @@ sub B::GV::savecv
    if ($name eq "bootstrap" && $cv->XSUB) 
     {
      my $file = $cv->FILEGV->SV->PV;
-     $bootstrap->add($file);
      my $name = $gv->STASH->NAME.'::'.$name;
      no strict 'refs';
      *{$name} = \&Dummy_BootStrap;   
+     $xsub{$gv->STASH->NAME}='Dynamic';
      $cv = $gv->CV;
     }
    warn sprintf("saving extra CV &%s::%s (0x%x) from GV 0x%x\n",
@@ -1262,7 +1279,7 @@ sub delete_unsaved_hashINC{
 	my $packname=shift;
 	$packname =~ s/\:\:/\//g;
 	$packname .= '.pm';
-	warn "deleting $packname" if $INC{$packname} ;# debug
+#	warn "deleting $packname" if $INC{$packname} ;# debug
 	delete $INC{$packname};
 }
 sub walkpackages 
@@ -1330,7 +1347,7 @@ sub save_main {
     my $init_av = init_av->save;
     $init->add(sprintf("PL_main_root = s\\_%x;", ${main_root()}),
 	       sprintf("PL_main_start = s\\_%x;", ${main_start()}),
-	       "PL_initav = $init_av;");
+              "PL_initav = (AV *) $init_av;");                                
     save_context();
     warn "Writing output\n";
     output_boilerplate();
@@ -1352,7 +1369,7 @@ sub init_sections {
 		    xpviv => \$xpvivsect, xpvnv => \$xpvnvsect,
 		    xpvmg => \$xpvmgsect, xpvlv => \$xpvlvsect,
 		    xrv => \$xrvsect, xpvbm => \$xpvbmsect,
-		    xpvio => \$xpviosect, bootstrap => \$bootstrap);
+		    xpvio => \$xpviosect);
     my ($name, $sectref);
     while (($name, $sectref) = splice(@sections, 0, 2)) {
 	$$sectref = new B::C::Section $name, \%symtable, 0;
