@@ -9,12 +9,16 @@ Thread::Specific - thread-specific keys
     use Thread::Specific;
     my $k = key_create Thread::Specific;
 
+=head1 DESCRIPTION
+
+C<key_create> returns a unique thread-specific key.
+
 =cut
 
 sub import {
     use attrs qw(locked method);
     require fields;
-    fields->import(@_);
+    fields::->import(@_);
 }	
 
 sub key_create {
