@@ -12,17 +12,17 @@ startsh='#!/bin/sh'
 
 archname='cygwin32'
 cc='gcc'
-libpth='/cygnus/cygwin-b20/H-i586-cygwin32/i586-cygwin32/lib /usr/local/lib'
+libpth='/usr/i586-cygwin32/lib /usr/lib /usr/local/lib'
 so='dll'
-libs='-lcygwin -lm -lc -lkernel32'
+libs='-lcygwin -lm -lkernel32'
 #optimize='-g'
-ccflags='-DCYGWIN32'
-ldflags='-L. -L/usr/local/lib'
+ccflags='-DCYGWIN32 -I/usr/include -I/usr/local/include'
+ldflags='-L/usr/i586-cygwin32/lib -L/usr/lib -L/usr/local/lib'
 usemymalloc='n'
 dlsrc='dl_cygwin32.xs'
 cccdlflags=' '
 ld='ld2'
-lddlflags='--export-dynamic -L. -L/usr/local/lib'
+lddlflags='-L/usr/local/lib'
 useshrplib='true'
 libperl='libperl.a'
 dlext='dll'
@@ -30,3 +30,7 @@ dlext='dll'
 man1dir=/usr/local/man/man1
 man3dir=/usr/local/man/man3
 sitelib=/usr/local/lib/perl5/site_perl
+
+case "$ldlibpthname" in
+'') ldlibpthname=PATH ;;
+esac
