@@ -7,6 +7,8 @@
  *
  */
 
+START_EXTERN_C
+
 #ifdef DOINIT
 EXTCONST unsigned char PL_utf8skip[] = {
 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, /* ascii */
@@ -22,6 +24,8 @@ EXTCONST unsigned char PL_utf8skip[] = {
 EXTCONST unsigned char PL_utf8skip[];
 #endif
 
+END_EXTERN_C
+
 #define IN_UTF8 (PL_curcop->op_private & HINT_UTF8)
 
 #define UTF8SKIP(s) PL_utf8skip[*(U8*)s]
